/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * main.c
 * ------
 *
 * Test that widget set!
 */

#include "impulse/rows/main.h"

#include "impulse/rows/constructor.h"
#include "impulse/rows/window.h"

#include "impulse/util/main.h"
#include "impulse/util/event.h"
#include "impulse/util/thread.h"

#include "gl_test.h"


impulse_bool timeout(impulse_event *ev, void *data)
{
  void **blah = data;
  ROWS_point *point = ROWS_child_get_origin(ROWS_child_cast(blah[0]));

  printf("\n\n ***** %s *****\n\n", blah[1]);
  printf("Origin X:%d Y:%d\n", MICRONS_TO_PIXEL(point->x), MICRONS_TO_PIXEL(point->y));
  point = ROWS_child_get_size(ROWS_child_cast(blah[0]));
  printf("Size X:%d Y:%d\n", MICRONS_TO_PIXEL(point->x), MICRONS_TO_PIXEL(point->y));
  point = ROWS_box_get_position(ROWS_box_cast(blah[0]));
  printf("Position X:%d Y:%d\n", MICRONS_TO_PIXEL(point->x), MICRONS_TO_PIXEL(point->y));
  point = ROWS_box_get_physical(ROWS_box_cast(blah[0]));
  printf("Physical X:%d Y:%d\n", MICRONS_TO_PIXEL(point->x), MICRONS_TO_PIXEL(point->y));
  point = ROWS_box_get_virtual(ROWS_box_cast(blah[0]));
  printf("Virtual X:%d Y:%d\n\n", MICRONS_TO_PIXEL(point->x), MICRONS_TO_PIXEL(point->y));
  return 1;
}

#define ACTUAL 1
#define NESTED 1

impulse_int main(impulse_int argc,
		 impulse_char *argv[])
{
  ROWS_widget window;
//  ROWS_widget layout1;
  ROWS_widget layout2;
  ROWS_widget child1;
  ROWS_widget child2;
  ROWS_widget child3;
  ROWS_widget child4;
  
  impulse_int glv1;
  impulse_int glv2;
  impulse_int glv3;
  impulse_int glh1;
  impulse_int glh2;
  impulse_int glh3;
  impulse_int glh4;  

  impulse_int glhm;
  impulse_int glvm;

  
  /* Start things off */
  ROWS_init(argc, argv);
  impulse_command_parse();

  /* The main window */
  window = ROWS_window_create("A guidelines test");

  /* FIXME: Drink. Girls! */
  ROWS_window_set_class(ROWS_window_cast(window), "fek off");
  ROWS_window_set_icon_name(ROWS_window_cast(window), "FatherJack");
  ROWS_child_set_visible(ROWS_child_cast(window),1);

  /* Create a few guidelines */
  glv1 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_VERTICAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 ROWS_CONSTRUCTOR_GL_LEFT, 
				 ROWS_CONSTRUCTOR_GL_RIGHT,
				 0.3333);
  glv2 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_VERTICAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 ROWS_CONSTRUCTOR_GL_LEFT, 
				 ROWS_CONSTRUCTOR_GL_RIGHT,
				 0.666);
  glh1 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_HORIZONTAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 ROWS_CONSTRUCTOR_GL_TOP, 
				 ROWS_CONSTRUCTOR_GL_BOTTOM,
				 0.25);
  
  glv3 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_VERTICAL,
				 ROWS_CONSTRUCTOR_ABSOLUTE,
				 glv1,
				 ROWS_CONSTRUCTOR_GL_LEFT,
				 MM_TO_MICRONS(1));
  glh2 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_HORIZONTAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 glh1,
				 ROWS_CONSTRUCTOR_GL_BOTTOM,
				 .25);
  glh3 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_HORIZONTAL,
				 ROWS_CONSTRUCTOR_ABSOLUTE,
				 glh2,
				 ROWS_CONSTRUCTOR_GL_BOTTOM,
				 MM_TO_MICRONS(3));
  glh4 = ROWS_constructor_add_GL(ROWS_constructor_cast(window),
				 ROWS_CONSTRUCTOR_HORIZONTAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 glh3,
				 ROWS_CONSTRUCTOR_GL_BOTTOM,
				 .5);

#if NESTED
  /* The nested constructor */
  layout2 = ROWS_constructor_create();
  ROWS_constructor_add_child(ROWS_constructor_cast(window),
			     ROWS_child_cast(layout2),
			     glv1, glv2, glh3, glh4);

#if ACTUAL
  ROWS_child_set_actualised(ROWS_child_cast(layout2),1);
#endif
  ROWS_child_set_visible(ROWS_child_cast(layout2), 1);
  
  /* Create a few guidelines */
  glhm = ROWS_constructor_add_GL(ROWS_constructor_cast(layout2),
				 ROWS_CONSTRUCTOR_HORIZONTAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 ROWS_CONSTRUCTOR_GL_TOP, 
				 ROWS_CONSTRUCTOR_GL_BOTTOM,
				 .5);
  glvm = ROWS_constructor_add_GL(ROWS_constructor_cast(layout2),
				 ROWS_CONSTRUCTOR_VERTICAL,
				 ROWS_CONSTRUCTOR_RELATIVE,
				 ROWS_CONSTRUCTOR_GL_LEFT, 
				 ROWS_CONSTRUCTOR_GL_RIGHT,
				 .5);
#endif
  /* Add some contents to the constructors */
  child1 = gl_test_create();
  ROWS_constructor_add_child(ROWS_constructor_cast(window),
			     ROWS_child_cast(child1),
			     glv1, glv2, 
			     glh1, glh2);
  ROWS_child_set_visible(ROWS_child_cast(child1),1);
#if ACTUAL
  ROWS_child_set_actualised(ROWS_child_cast(child1),1);
#endif

#if NESTED
  /* Add another gadget */
  child2 = gl_test_create();
  ROWS_constructor_add_child(ROWS_constructor_cast(layout2),
			     ROWS_child_cast(child2),
			     ROWS_CONSTRUCTOR_GL_LEFT, 
			     glvm, 
			     ROWS_CONSTRUCTOR_GL_TOP, 
			     glhm);
  ROWS_child_set_visible(ROWS_child_cast(child2),1);
# if ACTUAL
  ROWS_child_set_actualised(ROWS_child_cast(child2),1);
#endif
#endif
  
  /* Add some contents to the constructors */
  child3 = gl_test_create();
  ROWS_constructor_add_child(ROWS_constructor_cast(window),
			     ROWS_child_cast(child3),
			     glv2, 
			     ROWS_CONSTRUCTOR_GL_RIGHT, 
			     ROWS_CONSTRUCTOR_GL_TOP,
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child3),1);
#if ACTUAL
  ROWS_child_set_actualised(ROWS_child_cast(child3),1);
#endif

#if NESTED
  /* Add another gadget */
  child4 = gl_test_create();
  ROWS_constructor_add_child(ROWS_constructor_cast(layout2),
			     ROWS_child_cast(child4),
			     glvm, 
			     ROWS_CONSTRUCTOR_GL_RIGHT, 
			     ROWS_CONSTRUCTOR_GL_TOP, 
			     ROWS_CONSTRUCTOR_GL_BOTTOM);
  ROWS_child_set_visible(ROWS_child_cast(child4),1);
#if ACTUAL
  ROWS_child_set_actualised(ROWS_child_cast(child4),1);
#endif
#endif
  /* Do a little event processing */
  impulse_main();

  /* Good people always clean up after themselves */
  ROWS_cleanup();
  
  return 0;
}

/* End of file */
