/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * impulse_message.h
 * -----------------
 *
 * Communication transport functions
 */

#include "../../../config.h"

#include <impulse/comms/message.h>
#include <impulse/util/memory.h>


impulse_message impulse_message_create(impulse_ushort service,
				       impulse_ushort opcode,
				       impulse_ushort length)
{
  impulse_message msg = impulse_alloc(sizeof(struct _impulse_message));
  
  msg->service = service;
  msg->opcode = opcode;
  
  msg->header_length = sizeof(struct _impulse_message_header);
  msg->body_length = sizeof(impulse_ubyte) * length;
  msg->data_length = msg->header_length + msg->body_length;
  
  msg->data = impulse_alloc(msg->data_length);
  msg->header = msg->data;
  msg->body = msg->data + msg->header_length;

  if (IMPULSE_ENDIAN == IMPULSE_BIG_ENDIAN)
  {
    msg->header->service = service;
    msg->header->opcode = opcode;
    msg->header->length = length;
  }
  else
  {
    msg->header->service = impulse_ushort_swap(service);
    msg->header->opcode = impulse_ushort_swap(opcode);
    msg->header->length = impulse_ushort_swap(length);
  }
  
  return msg;
}


impulse_message impulse_message_create_header(impulse_message_header header)
{
  impulse_message msg = impulse_alloc(sizeof(struct _impulse_message));
  
  if (IMPULSE_ENDIAN == IMPULSE_BIG_ENDIAN)
  {
    msg->service = header->service;
    msg->opcode = header->opcode;
    msg->body_length = header->length;
  }
  else
  {
    msg->service = impulse_ushort_swap(header->service);
    msg->opcode = impulse_ushort_swap(header->opcode);
    msg->body_length = impulse_ushort_swap(header->length);
  }
  
  msg->header_length = sizeof(struct _impulse_message_header);
  msg->data_length = msg->header_length + msg->body_length;
  
  msg->data = impulse_alloc(msg->data_length);
  msg->header = msg->data;
  msg->body = msg->data + msg->header_length;

  return msg;
}


void impulse_message_destroy(impulse_message msg)
{
  impulse_free(msg->data);
  impulse_free(msg);
}


/* End of file */
