/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * brush.h
 * -------
 *
 * A widget used to draw on a drawable widget
 */

#ifndef __impulse_rows_brush_h
#define __impulse_rows_brush_h

#include <impulse/util/unistring.h>
#include <impulse/util/measure.h>
#include <impulse/rows/colour.h>

/*                            ======--------======                           */

/* Brush widget type */

typedef void **ROWS_brush;

/*                            ======--------======                           */

#include <impulse/rows/drawable.h>

/* Data types */

typedef struct _ROWS_brush_shape
{
  enum
  {
    ROWS_BRUSH_LINE,
    ROWS_BRUSH_CURVE
  } shape_type;
  microns x,  y;
  microns x1, y1;
} ROWS_brush_shape;

typedef struct _ROWS_brush_measure
{
  microns width;
  microns ascend, descend;

  microns left_overspill;
  microns right_overspill;
} ROWS_brush_measure;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget         ROWS_brush_create         (ROWS_drawable drawable);
extern ROWS_widget        *ROWS_brush_initialise     (int id,
						      ROWS_drawable drawable);
extern inline int          ROWS_brush_next_identifier(void);
extern inline ROWS_brush   ROWS_brush_cast           (void **widget);

/*                            ======--------======                           */

/* Class-specific functions */

/*
 * This function is /not/ thread-safe (use ROWS_widget_lock before
 * calling it)
 */
extern void ROWS_brush_set_gc        (ROWS_brush brush);
extern void ROWS_brush_use_private_gc(ROWS_brush brush);

/* Brush state functions */

extern void ROWS_brush_set_linewidth (ROWS_brush brush,
				      int        line_width);
extern void ROWS_brush_set_fgcolour  (ROWS_brush  brush,
				      ROWS_colour colour);
extern void ROWS_brush_set_bgcolour  (ROWS_brush  brush,
				      ROWS_colour colour);
extern int  ROWS_brush_get_fgpixel   (ROWS_brush brush);

/* Draw with this brush */

extern void ROWS_brush_draw_point    (ROWS_brush brush,
				      microns x, microns y);
extern void ROWS_brush_draw_line     (ROWS_brush brush,
				      microns x1, microns y1,
				      microns x2, microns y2);
extern void ROWS_brush_draw_rectangle(ROWS_brush brush,
				      microns x, microns y,
				      microns w, microns h);
extern void ROWS_brush_draw_arc      (ROWS_brush brush,
				      microns x, microns y,
				      microns w, microns h,
				      int angle1, int angle2);
extern void ROWS_brush_fill_rectangle(ROWS_brush brush,
				      microns x, microns y,
				      microns w, microns h);
extern void ROWS_brush_fill_arc      (ROWS_brush brush,
				      microns x, microns y,
				      microns w, microns h,
				      int angle1, int angle2);
extern void ROWS_brush_draw_points   (ROWS_brush  brush,
				      ROWS_point *points,
				      int         nopoints);
extern void ROWS_brush_draw_shape    (ROWS_brush        brush,
				      ROWS_brush_shape *points,
				      int               nopoints);

/* Font functions */

extern void        ROWS_brush_set_font          (ROWS_brush          brush,
						 char               *font);
extern void        ROWS_brush_set_font_size     (ROWS_brush          brush,
						 microns             sizex,
						 microns             sizey);
extern void        ROWS_brush_font_paint        (ROWS_brush          brush,
						 impulse_unistring   text,
						 microns             x,
						 microns             y);
extern void        ROWS_brush_font_paint_matrix (ROWS_brush          brush,
						 impulse_unistring   text,
						 microns             x,
						 microns             y,
						 float              *matrix);
extern void        ROWS_brush_font_measure      (ROWS_brush          brush,
						 impulse_unistring   text,
						 ROWS_brush_measure *result);
extern int         ROWS_brush_font_measure_split(ROWS_brush          brush,
						 impulse_unistring   text,
						 microns             maxwidth);
extern int         ROWS_brush_font_measure_wrap (ROWS_brush          brush,
					 	 impulse_unistring   text,
						 microns             maxwidth,
						 impulse_unichar     brk);

#endif
