/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * colour.h
 * --------
 *
 * ROWS colour manipulation (maybe this should be a widget, but
 * I don't want the overhead yet...)
 */

#ifndef __impulse_rows_colour_h
#define __impulse_rows_colour_h

#include <impulse/util/setup.h>

/*                            ======--------======                           */

/* Colour type */

struct _ROWS_colour_description
{
  enum ROWS_colour_type
  {
    /* Device-independant */
    ROWS_colour_XYZ,  /* The CIE colourspace */
    ROWS_colour_LUV,  /* Once again, CIE, but perceptually uniform */

    /* ROWS_colour_HVC - anyone know how to specify this? */

    /* Device-dependant */
    ROWS_colour_HSV,  /* Hue, Saturation, Value */
    ROWS_colour_HLS,  /* Hue, Light, Saturation */
    ROWS_colour_CMYK, /* Reflected light */
    ROWS_colour_RGB   /* Transmitted light */
  } type;

  union
  {
    struct { impulse_int x, y, z; }    xyz;
    struct { impulse_int l, u, v; }    luv;
    struct { impulse_int h, s, v; }    hsv;
    struct { impulse_int h, l, s; }    hls;
    struct { impulse_int c, m, y, k; } cmyk;
    struct { impulse_int r, g, b; }    rgb;
  } data;
};

typedef struct _ROWS_colour_description ROWS_colour_description;
typedef ROWS_colour_description ROWS_colour_desc;
typedef enum ROWS_colour_type ROWS_colour_type;

/*
 * Matrices contain values from 0-16384, colours are defined using
 * values from 0-1024
 */
struct _ROWS_gamut_rgb {
  impulse_int matrix[3][3];     /* [X,Y,Z] = matrix*[R,G,B] (auth) */

  impulse_int invmatrix[3][3];  /* Inverse of above, calculated */
  
  impulse_int r[2], g[2], b[2]; /* Colour coordinates (informative) */
  impulse_int C[3];             /* Luminances (RGB, informative) */
};

typedef struct _ROWS_gamut_rgb *ROWS_gamut_rgb;

struct _ROWS_colour {
  void **drawable;

  int x,y,z; /* (authoritative) CIE representation of this colour */
  int r,g,b; /* (informative) RGB equivalent */

  impulse_bool   is_spot;
  ROWS_gamut_rgb gamut;
  
  void *data;
};

typedef struct _ROWS_colour *ROWS_colour;

/*                            ======--------======                           */

/* Colour functions */

/*
 * Note: this creates a colour according to the device's default gamut
 * Use ROWS_colour_set_gamut_rgb if the colour was created on a
 * different device (or just use ROWS_colour_cie_*...)
 */
extern ROWS_colour ROWS_colour_for_drawable(void             **drawable,
					    ROWS_colour_desc   colour,
					    ROWS_gamut_rgb     gamut,
					    impulse_bool       make_spot);

extern ROWS_colour ROWS_colour_rgb_for_drawable(void **drawable,
						int r,
						int g,
						int b);
extern ROWS_colour ROWS_colour_cie_for_drawable(void **drawable,
						int x,
						int y,
						int z);

extern ROWS_colour ROWS_colour_create_pixel_colour(void  **drawable,
						   impulse_int pixel);
extern ROWS_colour ROWS_colour_named_for_drawable (void **drawable,
						   char *name);

extern void        ROWS_colour_destroy(ROWS_colour c);

/*                            ======--------======                           */

/* Colour space functions */

extern impulse_bool   ROWS_colour_is_programmable(void **drawable);
extern void           ROWS_colour_allocate_spread(void **drawable,
						  int    size);
extern void           ROWS_colour_set_gamut_rgb  (ROWS_colour    col,
						  ROWS_gamut_rgb gamut);
extern ROWS_gamut_rgb ROWS_colour_make_gamut_rgb (int Rx, int Ry,
						  int Gx, int Gy,
						  int Bx, int By,
						  int Cr,
						  int Cg,
						  int Cb);
extern impulse_bool   ROWS_colour_reset          (ROWS_colour      colour,
						  ROWS_colour_desc desc);

/* (Gamut can be NULL if converting between device-independant spaces) */
extern ROWS_colour_desc ROWS_colour_convert(ROWS_colour_desc from,
					    ROWS_colour_type to,
					    ROWS_gamut_rgb   gamut);

#endif
