/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * input.h
 * -------
 *
 * Mouse gesture description
 */

#ifndef __impulse_rows_input_h
#define __impulse_rows_input_h

/*                            ======--------======                           */

/* Mouse button types */

#if 0
/* Standard buttons */
#define ROWS_MOUSE_BUTTON_ALTERNATE 1
#define ROWS_MOUSE_BUTTON_MENU      2
#define ROWS_MOUSE_BUTTON_ACTION    4

/* Nonstandard buttons */
#define ROWS_MOUSE_BUTTON_ROLLUP    8
#define ROWS_MOUSE_BUTTON_ROLLDOWN  16
#endif

/* These are renamed left, right, middle, cos their
 * characteristics eg menu, action are assigned by
 * the PLAF. */

/* Standard buttons */
#define ROWS_MOUSE_BUTTON_LEFT     1
#define ROWS_MOUSE_BUTTON_MIDDLE   2
#define ROWS_MOUSE_BUTTON_RIGHT    3
#define ROWS_MOUSE_BUTTON_ROLLUP   4
#define ROWS_MOUSE_BUTTON_ROLLDOWN 5


/* Input device state */

/* Mouse button state */
#define ROWS_STATE_BUTTON_LEFT     1
#define ROWS_STATE_BUTTON_MIDDLE   2
#define ROWS_STATE_BUTTON_RIGHT    4
#define ROWS_STATE_BUTTON_ROLLUP   8
#define ROWS_STATE_BUTTON_ROLLDOWN 16

#define ROWS_STATE_BUTTONS         31

/* Key modifier state */
#define ROWS_STATE_KEY_SHIFT       32
#define ROWS_STATE_KEY_ALT         64
#define ROWS_STATE_KEY_CONTROL     128
/* Do we need to have the following?? Probably not worth worrying about */
#if 0 
#define ROWS_STATE_KEY_META        256
#define ROWS_STATE_KEY_SUPER       512
#define ROWS_STATE_KEY_HYPER       1024

#define ROWS_STATE_KEYS            2016
#else
#define ROWS_STATE_KEYS            224
#endif


/* Mouse gesture types */
/* THis is not used any more */
#if 0
typedef enum {
  ROWS_gesture_click,
  ROWS_gesture_doubleclick,
  ROWS_gesture_tripleclick,
  ROWS_gesture_drag
} ROWS_gesture_type;
#endif

/*                            ======--------======                           */

/* Keyboard input */

/* Key modifiers */

#define ROWS_INPUT_KEY 0xdb80

/* The locks */
#define ROWS_INPUT_KEY_SCROLL_LOCK 0xdc20
#define ROWS_INPUT_KEY_CAPS_LOCK   0xdc21
#define ROWS_INPUT_KEY_NUM_LOCK    0xdc22
#define ROWS_INPUT_KEY_SHIFT_LOCK  0xdc23

/* Navigation */

#define ROWS_INPUT_KEY_UP          0xdc30
#define ROWS_INPUT_KEY_DOWN        0xdc31
#define ROWS_INPUT_KEY_LEFT        0xdc32
#define ROWS_INPUT_KEY_RIGHT       0xdc33
#define ROWS_INPUT_KEY_HOME        0xdc34
#define ROWS_INPUT_KEY_END         0xdc35
#define ROWS_INPUT_KEY_PGUP        0xdc36
#define ROWS_INPUT_KEY_PGDOWN      0xdc37
#define ROWS_INPUT_KEY_PRIOR       0xdc38
#define ROWS_INPUT_KEY_BEGIN       0xdc39

/* Misc */

#define ROWS_INPUT_KEY_SYSRQ       0xdc40
#define ROWS_INPUT_KEY_PRINTSCREEN 0xdc41
#define ROWS_INPUT_KEY_PAUSE       0xdc42
#define ROWS_INPUT_KEY_BREAK       0xdc43
#define ROWS_INPUT_KEY_ESCAPE      0xdc44
#define ROWS_INPUT_KEY_INSERT      0xdc45
#define ROWS_INPUT_KEY_DELETE      0xdc46
#define ROWS_INPUT_KEY_WINDOWS     0xdc47
#define ROWS_INPUT_KEY_WINDOWS_R   0xdc48
#define ROWS_INPUT_KEY_MENU        0xdc49

#define ROWS_INPUT_KEY_CLEAR       0xdc4a
#define ROWS_INPUT_KEY_SELECT      0xdc4b
#define ROWS_INPUT_KEY_EXECUTE     0xdc4c
#define ROWS_INPUT_KEY_UNDO        0xdc4d
#define ROWS_INPUT_KEY_REDO        0xdc4e
#define ROWS_INPUT_KEY_FIND        0xdc4f
#define ROWS_INPUT_KEY_CANCEL      0xdc50
#define ROWS_INPUT_KEY_HELP        0xdc51
#define ROWS_INPUT_KEY_MODESWITCH  0xdc52

#endif
