/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * window.c
 * --------
 *
 * Top-level windows
 */

#include "../../../config.h"

#include "impulse/rows/window_int.h"

#include "impulse/util/thread.h"
#include "impulse/util/memory.h"


#if HAVE_GUI_XWIN
#include "impulse/rows/window_xwin.c"
#else
#error "Windows not implemented on this platform"
#endif

/*                            ======--------======                           */

extern void ROWS_window_int_setup  (ROWS_widget          *widget,
				    _p_ROWS_window_data  *data);
extern void ROWS_window_int_destroy(_p_ROWS_window_data **private);

static void ROWS_window_setup(ROWS_widget         *widget,
			      _p_ROWS_window_data *data);
static int  window_ID           = -1;
       int  _p_ROWS_window_type = -1;

static void ROWS_window_destroy(void **private);

/*                            ======--------======                           */

/* Standard widget functions */

ROWS_widget ROWS_window_create(char *title)
{
  ROWS_widget *wid;

  if (window_ID   == -1)
    window_ID   = ROWS_constructor_next_identifier();

  wid = ROWS_window_initialise(window_ID,
			       title);

  return *wid;
}

ROWS_widget *ROWS_window_initialise(int id,
				    char *title)
{
  ROWS_widget         *widget;
  _p_ROWS_window_data *data;

  if (window_ID   == -1)
    window_ID   = ROWS_constructor_next_identifier();
  if (_p_ROWS_window_type == -1)
    _p_ROWS_window_type = ROWS_widget_next_type();

  data = impulse_alloc(sizeof(_p_ROWS_window_data));

  /* Window is derived from a constructor */
  widget = ROWS_constructor_initialise(id);
  ROWS_widget_set_type(widget, window_ID, _p_ROWS_window_type);
  ROWS_widget_set_destructor(widget, window_ID, ROWS_window_destroy);

  ROWS_window_setup(widget, data);

  if (title != NULL)
    ROWS_window_set_title(ROWS_window_cast(*widget), title);

  return widget;
}

int ROWS_window_next_identifier(void)
{
  if (window_ID==-1)
    window_ID = ROWS_constructor_next_identifier();

  return window_ID + 1;
}

ROWS_window ROWS_window_cast(void **widget)
{
  if (window_ID   == -1)
    window_ID   = ROWS_constructor_next_identifier();
  if (_p_ROWS_window_type == -1)
    _p_ROWS_window_type = ROWS_widget_next_type();

  return ROWS_widget_cast_misc(widget, window_ID, _p_ROWS_window_type);
}

void ROWS_window_setup(ROWS_widget         *widget,
		       _p_ROWS_window_data *data)
{
  _p_ROWS_window_data **private;
  ROWS_child        child;

  const char *default_title = "<< Untitled ROWS window >>";
  const char *default_class = "window";
  const char *icon_name = "<< Untitled >>";

  private  = (_p_ROWS_window_data **) ROWS_widget_private(widget, window_ID);
  *private = data;

  (*private)->properties.title         = impulse_alloc(strlen(default_title)+1);
  (*private)->properties.class         = impulse_alloc(strlen(default_class)+1);
  (*private)->properties.icon_name     = impulse_alloc(strlen(icon_name)+1);

  strcpy((*private)->properties.title, default_title);
  strcpy((*private)->properties.class, default_class);
  strcpy((*private)->properties.icon_name, icon_name);

  (*private)->properties.min_size.x    = 50;
  (*private)->properties.min_size.y    = 50;
  (*private)->properties.type          = ROWS_WINDOW_STANDARD;
  (*private)->properties.own_colourmap = 0;

  (*private)->properties.attributes    =
    ROWS_WINDOW_TITLE      |
    ROWS_WINDOW_RESIZE     |
    ROWS_WINDOW_CLOSE      |
    ROWS_WINDOW_TOGGLESIZE |
    ROWS_WINDOW_SLIPPERY;
  
  child = ROWS_child_cast(*widget);

  ROWS_window_int_setup(widget, data);
}


void ROWS_window_destroy(void **data)
{
  _p_ROWS_window_data **priv;

  priv  = (_p_ROWS_window_data **) data;

  ROWS_window_int_destroy(priv);
  
  impulse_free((*priv)->properties.title);
  impulse_free((*priv)->properties.class);
  impulse_free((*priv)->properties.icon_name);

  impulse_free(*priv);
}


/* End of file */
