/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * queue.h
 * -------
 *
 * Double headed queues
 */

#ifndef __impulse_util_queue_h
#define __impulse_util_queue_h

#include <impulse/util/iterator.h>

#ifdef __cplusplus
extern "C" {  
#endif

/*                            ======--------======                           */

/* queueing data structures */

typedef struct _impulse_queue *impulse_queue;
typedef void (*impulse_queue_cleaner_func)(void *data);

/*                            ======--------======                           */

/* queueing functions */

impulse_queue impulse_queue_create(void);
void impulse_queue_destroy(impulse_queue queue);

void impulse_queue_cleaner(impulse_queue queue,
			   impulse_queue_cleaner_func func);
impulse_iterator impulse_queue_elements(impulse_queue queue);

impulse_bool impulse_queue_empty(impulse_queue queue);
impulse_uint impulse_queue_count(impulse_queue queue);

void impulse_queue_enqueue_end(impulse_queue queue,
			       void *data,
			       impulse_bool head);
void *impulse_queue_serve_end(impulse_queue queue,
			      impulse_bool head);

#define impulse_queue_enqueue(queue, data) impulse_queue_enqueue_end(queue, data, 0)
#define impulse_queue_serve(queue) impulse_queue_serve_end(queue, 1)

void impulse_queue_remove(impulse_queue queue,
			  void *data);
void impulse_queue_clear(impulse_queue queue);
impulse_bool impulse_queue_contains(impulse_queue queue,
				    void *data);

#ifdef __cplusplus
}
#endif

#endif

