dnl
dnl Macro to determine which thread library loading subsystem
dnl to use
dnl

dnl *****************************************************************
dnl * Command line arguments for multi-thread support
dnl *****************************************************************

AC_DEFUN(IMPULSE_THREADS_INIT,
[
  AC_ARG_ENABLE([threads],
  [  --enable-threads        Enable the multi-threading interface [default=yes]],
  [
    if test "$enableval" = "yes" ; then
      USE_THREADS="1"
    else
      USE_THREADS="0"
    fi
  ],[
    USE_THREADS="1"
  ])

  AC_ARG_WITH([threads],
  [  --with-threads=[posix/pth/sun/win32]
                          Select the multi-threading interface],
  [
    case $with_threads in
     posix)
       THREADS_PREFERENCE="POSIX";;
       
     sun)
       THREADS_PREFERENCE="SUN";;
       
     win32)
       THREADS_PREFERENCE="WIN32";;
       
     pth)
       THREADS_PREFERENCE="PTH";;
       
     *)
       THREADS_PREFERENCE="NONE";;
   esac
  ],[
    THREADS_PREFERENCE="NONE";
  ])

  IMPULSE_THREADS_INCLUDE2=""
  IMPULSE_THREADS_INCLUDE1=""
  
  IMPULSE_MUTEX_CREATE_STATIC="0"
  IMPULSE_COND_CREATE_STATIC="0"
  IMPULSE_RWLOCK_CREATE_STATIC="0"
  IMPULSE_SEMA_CREATE_STATIC="0"
  
  IMPULSE_THREAD_IMPL="int"
  IMPULSE_MUTEX_IMPL="int"
  IMPULSE_COND_IMPL="int"
  IMPULSE_RWLOCK_IMPL="int"
  IMPULSE_SEMA_IMPL="int"
  
  THREADS_LIBRARY="none"
])



dnl *****************************************************************
dnl * Checking which multi-thread implementation is availble
dnl *****************************************************************

AC_DEFUN(IMPULSE_THREADS_CHECK,
[
  IMPULSE_MSG_SECTION([Checking multi-threading support])
  
  if test "$USE_THREADS" = "1" ; then
    if test "$THREADS_LIBRARY" = "none" ; then
      if test "$THREADS_PREFERENCE" = "NONE" -o "$THREADS_PREFERENCE" = "POSIX" ; then
        IMPULSE_THREADS_CHECK_POSIX
      fi
    fi
    if test "$THREADS_LIBRARY" = "none" ; then
      if test "$THREADS_PREFERENCE" = "NONE" -o "$THREADS_PREFERENCE" = "SUN" ; then
        IMPULSE_THREADS_CHECK_SUN
      fi
    fi
    if test "$THREADS_LIBRARY" = "none" ; then
      if test "$THREADS_PREFERENCE" = "NONE" -o "$THREADS_PREFERENCE" = "WIN32" ; then
        IMPULSE_THREADS_CHECK_WIN32
      fi
    fi
    if test "$THREADS_LIBRARY" = "none" ; then
      if test "$THREADS_PREFERENCE" = "NONE" -o "$THREADS_PREFERENCE" = "PTH" ; then
        IMPULSE_THREADS_CHECK_PTH
      fi
    fi
  
    dnl We still haven't found any, so lets disable threads
    if test "$THREADS_LIBRARY" = "none" ; then
      USE_THREADS="0"
    fi
  fi
  
  if test "$USE_THREADS" = "1" ; then
    GLOBAL_CFLAGS="$GLOBAL_CFLAGS -D_REENTRANT"
  fi
])


dnl *****************************************************************
dnl * Test for basic availability of POSIX threads
dnl *****************************************************************

AC_DEFUN(IMPULSE_THREADS_CHECK_POSIX,
[
  IMPULSE_PARAMS_SAVE
  
  POSIX_LIBS=""
  POSIX_CFLAGS=""
  POSIX_LDFLAGS=""
  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $THREADS_CFLAGS"
  LIBS="$CORE_LIBS $THREADS_LIBS"
  LDFLAGS="$CORE_LDFLAGS $THREADS_LDFLAGS"
	    
  AC_CHECK_LIB(pthread, pthread_create,
  [
    AC_CHECK_HEADER(pthread.h,
    [
      AC_CHECK_HEADER(semaphore.h,
      [
        AC_CHECK_LIB(pthread, sem_wait,
	[
	  THREADS_LIBRARY="posix"
	  AC_DEFINE(HAVE_THREADS_POSIX, 1)
	  POSIX_LIBS="-lpthread"
	],[
	  AC_CHECK_LIB(posix4, sem_wait,
	  [
	    THREADS_LIBRARY="posix"
	    AC_DEFINE(HAVE_THREADS_POSIX, 1)
	    POSIX_LIBS="-lpthread -lposix4"
	  ])
	])
      ])
    ])
  ])

  if test "$THREADS_LIBRARY" != "posix"; then
    dnl FreeBSD doesn't use the -lpthread library, but rather has a special
    dnl -pthread option to gcc

    AC_MSG_CHECKING([for -pthread flag to C compiler])
    LDFLAGS="$LDFLAGS -pthread"
    CFLAGS="$CFLAGS -pthread"
    AC_TRY_LINK([
      #include <pthread.h>
    ],
    [
      int i;
      pthread_t thread;
      
      pthread_create(&thread, NULL, NULL, NULL);
    ],
    [
      AC_MSG_RESULT([yes (probably FreeBSD)])
      THREADS_LIBRARY="posix"
      AC_DEFINE(HAVE_THREADS_POSIX, 1)
      POSIX_LDFLAGS="-pthread"
      POSIX_CFLAGS="-pthread"
    ],
    [
      AC_MSG_RESULT(no)
    ])
  fi

  IMPULSE_PARAMS_RESTORE
  if test "$THREADS_LIBRARY" = "posix" ; then
    THREADS_CFLAGS="$THREADS_CFLAGS $POSIX_CFLAGS"
    THREADS_LDFLAGS="$THREADS_LDFLAGS $POSIX_LDFLAGS"
    THREADS_LIBS="$THREADS_LIBS $POSIX_LIBS"
    
    IMPULSE_THREADS_CHECK_POSIX_OPT
  fi
])



dnl *****************************************************************
dnl * Check for GNU pth
dnl *****************************************************************


AC_DEFUN(IMPULSE_THREADS_CHECK_PTH,
[
  IMPULSE_PARAMS_SAVE
  PTH_LIBS=""
  PTH_CFLAGS=""
  PTH_LDFLAGS=""
  
  AC_PATH_PROG(PTH_CONFIG, "pth-config", "no")
  if test "$PTH_CONFIG" != "no" ; then
    PTH_CFLAGS=`$PTH_CONFIG $pth_config_args --cflags`
    PTH_LIBS=`$PTH_CONFIG $pth_config_args --libs`
    PTH_LDFLAGS=`$PTH_CONFIG $pth_config_args --ldflags`

    CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $THREADS_CFLAGS $PTH_CFLAGS"
    LIBS="$CORE_LIBS $THREADS_LIBS $PTH_LIBS"
    LDFLAGS="$CORE_LDFLAGS $THREADS_LDFLAGS $PTH_LDFLAGS"
    
    AC_CACHE_CHECK([for pth], impulse_cv_pth,
    [
      AC_TRY_COMPILE([
        #include <pth.h>
      ],[
        pth_init();
      ],[
        AC_TRY_LINK([
          #include <pth.h>
        ],[
	  pth_init();
	],[
          impulse_cv_pth="yes"
	],[
	  impulse_cv_pth="no"
	])
      ],[
        impulse_cv_pth="no"
      ])
    ])
      
    if test "$impulse_cv_pth" = "yes" ; then
      THREADS_LIBRARY="pth"
      AC_DEFINE(HAVE_THREADS_PTH, 1)
    fi
  fi
  
  IMPULSE_PARAMS_RESTORE
  if test "$THREADS_LIBRARY" = "pth" ; then
    THREADS_CFLAGS="$THREADS_CFLAGS $PTH_CFLAGS"
    THREADS_LDFLAGS="$THREADS_LDFLAGS $PTH_LDFLAGS"
    THREADS_LIBS="$THREADS_LIBS $PTH_LIBS"
    
    IMPULSE_THREADS_INCLUDE1="#include <pth.h>"
    IMPULSE_THREAD_IMPL="pth_t"
    IMPULSE_MUTEX_IMPL="pth_mutex_t"
    IMPULSE_COND_IMPL="pth_cond_t"
    IMPULSE_RWLOCK_IMPL="pth_rwlock_t"
    IMPULSE_SEMA_IMPL="struct _impulse_sema_impl { pth_mutex_t mutex; pth_cond_t cond; impulse_int value; }"
    
    IMPULSE_MUTEX_CREATE_STATIC="PTH_MUTEX_INIT"
    IMPULSE_COND_CREATE_STATIC="PTH_COND_INIT"
    IMPULSE_RWLOCK_CREATE_STATIC="PTH_RWLOCK_INIT"
    IMPULSE_SEMA_CREATE_STATIC="{ PTH_MUTEX_INIT, PTH_COND_INIT, 0 }"
  fi
])


dnl *****************************************************************
dnl * Check for SUN native threads
dnl *****************************************************************


AC_DEFUN(IMPULSE_THREADS_CHECK_SUN,
[
  IMPULSE_PARAMS_SAVE
  SUN_LIBS=""
  SUN_CFLAGS=""
  SUN_LDFLAGS=""
  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $THREADS_CFLAGS"
  LIBS="$CORE_LIBS $THREADS_LIBS"
  LDFLAGS="$CORE_LDFLAGS $THREADS_LDFLAGS"
      
  AC_CHECK_LIB(thread, thr_create,
  [
    AC_CHECK_HEADER(thread.h,
    [
      AC_CHECK_HEADER(synch.h,
      [
        THREADS_LIBRARY="sun"
	AC_DEFINE(HAVE_THREADS_SUN, 1)
	SUN_LIBS="-lthread"
      ])
    ])
  ])

  IMPULSE_PARAMS_RESTORE
  if test "$THREADS_LIBRARY" = "SUN" ; then
    THREADS_CFLAGS="$THREADS_CFLAGS $SUN_CFLAGS"
    THREADS_LDFLAGS="$THREADS_LDFLAGS $SUN_LDFLAGS"
    THREADS_LIBS="$THREADS_LIBS $SUN_LIBS"
    
    IMPULSE_THREADS_INCLUDE1="#include <thread.h>"
    IMPULSE_THREADS_INCLUDE2="#include <synch.h>"
    IMPULSE_THREAD_IMPL="thread_t"
    IMPULSE_MUTEX_IMPL="mutex_t"
    IMPULSE_COND_IMPL="cond_t"
    IMPULSE_RWLOCK_IMPL="rwlock_t"
    IMPULSE_SEMA_IMPL="sema_t"
    IMPULSE_MUTEX_CREATE_STATIC="DEFAULTMUTEX"
    IMPULSE_COND_CREATE_STATIC="DEFAULTCV"
    IMPULSE_RWLOCK_CREATE_STATIC="DEFAULTRWLOCK"
    IMPULSE_SEMA_CREATE_STATIC="DEFAULTSEMA"
  fi
])


dnl *****************************************************************
dnl * Check for WIN32 native threads
dnl *****************************************************************


AC_DEFUN(IMPULSE_THREADS_CHECK_WIN32,
[
  IMPULSE_PARAMS_SAVE
  WIN32_LIBS=""
  WIN32_CFLAGS=""
  WIN32_LDFLAGS=""
  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $THREADS_CFLAGS"
  LIBS="$CORE_LIBS $THREADS_LIBS"
  LDFLAGS="$CORE_LDFLAGS $THREADS_LDFLAGS"
	
  AC_CHECK_HEADER(windows.h,
  [
    THREADS_LIBRARY="win32"
    AC_DEFINE(HAVE_THREADS_WIN32)
  ])

  IMPULSE_PARAMS_RESTORE
  if test "$THREADS_LIBRARY" = "WIN32" ; then
    THREADS_CFLAGS="$THREADS_CFLAGS $WIN32_CFLAGS"
    THREADS_LDFLAGS="$THREADS_LDFLAGS $WIN32_LDFLAGS"
    THREADS_LIBS="$THREADS_LIBS $WIN32_LIBS"
    
    IMPULSE_THREADS_INCLUDE1="#include <windows.h>"
    IMPULSE_THREAD_IMPL="HANDLE"
    IMPULSE_MUTEX_IMPL="HANDLE"
    IMPULSE_COND_IMPL="struct _impulse_cond_impl { impulse_int waiters; HANDLE waiters_mutex; HANDLE sema; }"
    IMPULSE_RWLOCK_IMPL="struct _impulse_rwlock_impl { impulse_int counter; impulse_int nreaders; impulse_int nwriters; HANDLE mutex; HANDLE readers; HANDLE writers; }"
    IMPULSE_SEMA_IMPL="HANDLE"
    
    IMPULSE_MUTEX_CREATE_STATIC="0"
    IMPULSE_COND_CREATE_STATIC="{ 0, 0, 0 }"
    IMPULSE_RWLOCK_CREATE_STATIC="{ 0, 0, 0, 0 }"
    IMPULSE_SEMA_CREATE_STATIC="0"
  fi
])


dnl *****************************************************************
dnl * Various optional/non-portable Posix Threads features
dnl *****************************************************************


AC_DEFUN(IMPULSE_THREADS_CHECK_POSIX_OPT,
[
  IMPULSE_PARAMS_SAVE
  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $THREADS_CFLAGS"
  LDFLAGS="$CORE_LDFLAGS $THREADS_LDFLAGS"
  LIBS="$CORE_LIBS $THREADS_LIBS"

  AC_CHECK_FUNC(pthread_rwlock_init,
  [
    THREADS_POSIX_RWLOCK=1
  ],[
    THREADS_POSIX_RWLOCK=0
  ])


  AC_CACHE_CHECK([for recursive mutexes], impulse_cv_recursivemutex,
  [
    AC_TRY_LINK([
      #include <pthread.h>
    ],[
      pthread_mutexattr_t attr;
      pthread_mutexattr_init(&attr);
      pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
    ],[
      impulse_cv_recursivemutex="yes"
    ],[
      impulse_cv_recursivemutex="no"
    ])
  ])
  
  if test "$impulse_cv_recursivemutex" = "yes" ; then
    THREADS_POSIX_RECURSIVE=1
  else
    THREADS_POSIX_RECURSIVE=0
  fi


  AC_CACHE_CHECK([for errorchecking mutexes], impulse_cv_errorcheckingmutex,
  [
    AC_TRY_LINK([
      #include <pthread.h>
    ],[
      pthread_mutexattr_t attr;
      pthread_mutexattr_init(&attr);
      pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
    ],[
      impulse_cv_errorcheckingmutex="yes"
    ],[
      impulse_cv_errorcheckingmutex="no"
    ])
  ])

  if test "$impulse_cv_errorcheckingmutex" = "yes" ; then
    THREADS_POSIX_ERRORCHECKING=1
  else
    THREADS_POSIX_ERRORCHECKING=0
  fi

  if test "$THREADS_POSIX_RECURSIVE" = "0" -o "$THREADS_POSIX_ERRORCHECKING" = "0" ; then
    AC_CACHE_CHECK([for linux threads non-portable attributes], impulse_cv_linuxthreads,
    [
      AC_TRY_LINK([
        #include <pthread.h>
      ],[
        pthread_mutexattr_t attr;
        pthread_mutexattr_init(&attr);
        pthread_mutexattr_setkind_np(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
        pthread_mutexattr_setkind_np(&attr, PTHREAD_MUTEX_ERRORCHECK_NP);
      ],[
        impulse_cv_linuxthreads="yes"
      ],[
        impulse_cv_linuxthreads="no"
      ])
    ])

    if test "$impulse_cv_linuxthreads" = "yes" ; then
      THREADS_POSIX_LINUX_NP=1
    else 
      THREADS_POSIX_LINUX_NP=0
    fi
  fi

  if test "$THREADS_POSIX_LINUX_NP" = "1" ; then
    AC_DEFINE(HAVE_THREADS_POSIX_LINUX_NP, 1)
  fi
  if test "$THREADS_POSIX_RECURSIVE" = "1" ; then
    AC_DEFINE(HAVE_THREADS_POSIX_RECURSIVE, 1)
  fi
  if test "$THREADS_POSIX_RWLOCK" = "1" ; then
    AC_DEFINE(HAVE_THREADS_POSIX_RWLOCK, 1)
  fi
  if test "$THREADS_POSIX_ERRORCHECKING" = "1" ; then
    AC_DEFINE(HAVE_THREADS_POSIX_ERRORCHECKING, 1)
  fi
  
  IMPULSE_THREADS_INCLUDE1="#include <pthread.h>"
  IMPULSE_THREADS_INCLUDE2="#include <semaphore.h>"
  IMPULSE_THREAD_IMPL="pthread_t"
  
  if test "$THREADS_POSIX_LINUX_NP" = "0" -a "$THREADS_POSIX_RECURSIVE" = "0" ; then
    IMPULSE_MUTEX_IMPL="struct _impulse_mutex_impl { pthread_mutex_t mutex_native; impulse_bool is_recursive; pthread_mutex_t mutex_recursive; pthread_t locked_by; impulse_int lock_level; }"
    IMPULSE_MUTEX_CREATE_STATIC="{ PTHREAD_MUTEX_INITIALIZER, 0, PTHREAD_MUTEX_INITIALIZER, 0, 0 }"
  else
    IMPULSE_MUTEX_IMPL="pthread_mutex_t"
    IMPULSE_MUTEX_CREATE_STATIC="PTHREAD_MUTEX_INITIALIZER"
  fi
  
  IMPULSE_COND_IMPL="pthread_cond_t"
  IMPULSE_COND_CREATE_STATIC="PTHREAD_COND_INITIALIZER"
  
  if test "$THREADS_POSIX_RWLOCK" = "1" ; then
    IMPULSE_RWLOCK_IMPL="pthread_rwlock_t"
    IMPULSE_RWLOCK_CREATE_STATIC="PTHREAD_RWLOCK_INITIALIZER"
  else
    IMPULSE_RWLOCK_IMPL="struct _impulse_rwlock_impl { pthread_mutex_t mutex; pthread_cond_t notify; impulse_int readlocks; impulse_int writelocks; }"
    IMPULSE_RWLOCK_CREATE_STATIC="{ PTHREAD_MUTEX_INITIALIZER, PTHREAD_COND_INITIALIZER, 0, 0 }"
  fi
  
  IMPULSE_SEMA_IMPL="sem_t *"
  IMPULSE_SEMA_CREATE_STATIC="NULL"
  
  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Substitute all the variables we need
dnl *****************************************************************


AC_DEFUN(IMPULSE_THREADS_FINISH,
[
  AC_SUBST(USE_THREADS)
  AC_SUBST(THREADS_CFLAGS)
  AC_SUBST(THREADS_LDFLAGS)
  AC_SUBST(THREADS_LIBS)

  AC_SUBST(IMPULSE_THREADS_INCLUDE1)
  AC_SUBST(IMPULSE_THREADS_INCLUDE2)

  AC_SUBST(IMPULSE_MUTEX_CREATE_STATIC)
  AC_SUBST(IMPULSE_COND_CREATE_STATIC)
  AC_SUBST(IMPULSE_RWLOCK_CREATE_STATIC)
  AC_SUBST(IMPULSE_SEMA_CREATE_STATIC)
  
  AC_SUBST(IMPULSE_THREAD_IMPL)
  AC_SUBST(IMPULSE_MUTEX_IMPL)
  AC_SUBST(IMPULSE_COND_IMPL)
  AC_SUBST(IMPULSE_RWLOCK_IMPL)
  AC_SUBST(IMPULSE_SEMA_IMPL)
])


dnl *****************************************************************
dnl * Print out summary of multi-thread configuration
dnl *****************************************************************


AC_DEFUN(IMPULSE_THREADS_SUMMARY,
[
  IMPULSE_ECHO(["Threads library: $THREADS_LIBRARY"])
])

dnl End of file
