%{
#include <stdio.h>
#include <string.h>
#include "ROWSdoc_yacc.h"

#define STRING_NEWCOPY(x) strcpy(malloc(strlen(x)+1), x);

#define printf(x) /* */
%}

WHITESPACE    [ \n\t]+
IDENTIFIER    [A-z_][A-z_0-9]*
NUMBER        [0-9]+

PREWHITESPACE ([ \t]|(\\\n))+

%s preprocess
%s processparam
%s predefine
%s predefinename
%s predefineparam
%s comment
%s doccomment

%%

<INITIAL>^[\t ]*\#             printf("PREPROCESS\n"); BEGIN(preprocess); return PREPRO;
<INITIAL>typedef               printf("TYPEDEF\n"); return TYPEDEF;
<INITIAL>(struct)|(union)      printf("STRUCT\n"); return STRUCT;
<INITIAL>enum                  printf("ENUM\n"); return ENUM;

<INITIAL>extern                printf("EXTERN\n"); return EXTERN;
<INITIAL>static                printf("STATIC\n"); return STATIC;

<INITIAL>void                  printf("VOID\n"); return VOID;
<INITIAL>{IDENTIFIER}        { printf("ID\n");
			       yylval.string = STRING_NEWCOPY(yytext);
			       return IDENTIFIER;
                             }
<INITIAL>\/\*                  BEGIN(comment); return COMMENTSTART;
<INITIAL>\/\*\*                BEGIN(doccomment); return DOCCOMMENTSTART;
<INITIAL>{WHITESPACE}          printf("WS\n"); return WHITESPACE;
<INITIAL>{NUMBER}              yylval.numeric = atoi(yytext); return NUMBER;

<preprocess>define             BEGIN(predefine); return PREDEFINE;
<preprocess>if                 BEGIN(processparam); return PREIF;
<preprocess>(ifdef)|(ifndef)   BEGIN(processparam); return PREIFDEF;
<preprocess>pragma             BEGIN(processparam); return PREPRAGMA;
<preprocess>endif              BEGIN(processparam); return PREENDIF;
<preprocess>(undef)|(include)  BEGIN(processparam); return PREUNDEF; /* FIXME... (undef)|(include)??? Lazy, lazy, lazy ;-) */
<preprocess>else               BEGIN(processparam); return PREELSE;

<processparam>([^ \t\n\\]|(\\[^\n]))+  { printf("PREID\n");
		                 yylval.string = STRING_NEWCOPY(yytext);
		                 return IDENTIFIER;
			       }
<processparam>{PREWHITESPACE}    printf("PREWS\n"); return WHITESPACE;
<processparam>\n                 printf("PREEOLN\n"); BEGIN(0); return PREEOLN;

<predefine>{PREWHITESPACE}       printf("PD-WS\n"); BEGIN(predefinename); return WHITESPACE;
<predefinename>{IDENTIFIER}    { printf("PD-NAME\n");
			         yylval.string = STRING_NEWCOPY(yytext);
			         return IDENTIFIER;
			       }
<predefinename>\(                printf("PD-PARAMSTART\n"); BEGIN(predefineparam); return yytext[0];
<predefinename>[ \t]+            printf("PD-SPACE\n"); BEGIN(processparam); return WHITESPACE;
<predefinename>.               { printf("PD-MISC\n");
			         yylval.string = STRING_NEWCOPY(yytext);
			         BEGIN(processparam);
				 return IDENTIFIER;
			       }

<predefineparam>{PREWHITESPACE}  printf("PDP-WS\n"); return WHITESPACE;
<predefineparam>{IDENTIFIER}   { printf("PDP-ID\n");
			         yylval.string = STRING_NEWCOPY(yytext);
			         return IDENTIFIER;
			       }
<predefineparam>\)               printf("PDP-CLOSEBRACKET\n"); BEGIN(processparam); return yytext[0];

<predefine>[\n]                  BEGIN(0); return PREEOLN;
<predefinename>[\n]              BEGIN(0); return PREEOLN;
<predefineparam>[\n]             BEGIN(0); return PREEOLN;

<comment>([^\*\/]|(\*[^\/]))*    printf("BUMPH\n"); return COMMENTBUMPH;
<comment>\*\/                    printf("EOC\n"); BEGIN(0); return COMMENTEND;
<comment>.                       return COMMENTBUMPH;

<doccomment>\*\/                 BEGIN(0); return DCEND;
<doccomment>@{IDENTIFIER}        {
				   yylval.string = STRING_NEWCOPY(&yytext[1]);
				   return DCTOKEN;
				 }
<doccomment>^[ \t]*\*[ \t]+      return DCWHITESPACE;
<doccomment>^[ \t]+              return DCWHITESPACE;
<doccomment>[ \t]+               return DCWHITESPACE;
<doccomment>\n                   return DCWHITESPACE;
<doccomment>[^ \t\n]*            {
				   yylval.string = STRING_NEWCOPY(yytext);
				   return DCWORD;
				 }

<*>.                             return yytext[0];
%%


