/* Written by Andrew Hunter. 22/02/99 (That was when this comment was written ;-) */

/* Todo list:
 * 
 * Key: (-) - todo, (/) - partially done, (*) - done
 * 
 * * Deal with struct foo { } & struct { } declarations
 * * Deal with naked struct/enum definitions (not typedefs)
 * * Do something useful with naked struct/enum definitions... (Not blech or 
 *   hurrrgh, that is)
 * * Make enum return useful information
 * * Deal with enum foo { } & enum { } declarations
 * / Produce sensible output 
 * - fix #define foo bar\bar thing
 * - make #define foo(x) return useful information
 * * find out what's with the %nonassoc crap re VOID & WHITESPACE
 * * Produce more output on function declarations
 * - Write functions to properly free type descriptors (_s_typeidentifier)
 * * Make NUMBER return a value
 * * Fix #define foo_bar_baz            baz_foo_bar bug
 * - Distinguish unions & structs
 * * typedef foo bar   ; needs fixing
 * * Deal with comments in structs
 * * Deal with comments in enums
 * - Fix int x,y,z style variable declarations
 * * Fix int x,y,z style declarations in structs
 * - Think of something to do with preprocessor directives in structs/enums
 * - Do something useful with DOCCOMMENTS
 * * Go to bed
 */

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define YYERROR_VERBOSE
#define YYDEBUG 1
#define STRING_NEWCOPY(x) strcpy(malloc(strlen(x)+1), x);

#define IS_STRUCT   1
#define IS_ENUM     2
#define IS_FUNCTION 4
#define HAS_NAME    8
%}

%union {
  char *string;

  struct _s_simpletype {
    char *name;

    int is_flags;

    void *extra_data;
  } simpletype;

  struct _s_typeidentifier {
    char *type;
    int   pointerlevel;
    char *name;

    int   is_flags;
    void *extra_data;
  } type;

  struct _s_typepointer {
    char *type;
    int   pointerlevel;
  } typepointer;

  struct _s_structdef {
    struct _s_typeidentifier  id;
    struct _s_structdef      *next;
  } *structdef;

  struct _s_idlist {
    char             *name;
    struct _s_idlist *next;
  } *idlist;

  struct _s_param {
    struct _s_typeidentifier typepointer;

    struct _s_param       *next;
  } *parameters;

  struct _s_enum {
    int   number;
    char *name;
    
    struct _s_enum *next;
  } *enumt;

  struct _s_enumline {
    int   number;
    char *identifier;
    int   is_set;
   } enumline;

  struct _s_functiondef {
    struct _s_typeidentifier  returntype;
    struct _s_param          *parameters;
  } functiondef;

  int numeric;

  /* Actual definition of the header file */
  struct _s_functiondecl {
    struct _s_typeidentifier  type_name;
    struct _s_param          *parameters;
  } functiondecl;

  struct _s_variabledecl {
    struct _s_typeidentifier  type_name;
  } variabledecl;

  struct _s_typedefdecl {
    struct _s_typeidentifier  type_name;
  } typedefdecl;

  /* The headerfile itself */
  struct _s_headerfile {
    enum {
      DECL_FUNCTION,
      DECL_VARIABLE,
      DECL_NAKEDSTRUCT,
      DECL_TYPEDEF,
      DECL_DEFINE
    } headertype;

    union {
      struct _s_functiondecl function;
      struct _s_variabledecl variable;
      struct _s_variabledecl nakedstruct;
      struct _s_typedefdecl  typedefdecl;
    } data;

    struct _s_headerfile *next;
  } *headerfile;
}

%{
void output_parameters(struct _s_param *param)
{
  if (param==NULL)
    return;

  printf("  Parameter: %s (type: %s, pointerlevel %i)\n", param->typepointer.name,
         param->typepointer.type, param->typepointer.pointerlevel);
  output_parameters(param->next);
}

void free_parameters(struct _s_param *param)
{
  struct _s_param *next, *last;
    
  last = param;
  while (last!=NULL)
  {
    next = last->next;
    free(last->typepointer.name);
    free(last->typepointer.type);
    free(last);
    last = next;
  }
}

void output_type(struct _s_typeidentifier *type, int indent_level)
{
  int x;

  if (indent_level > 0)
  {
    for (x=0;x<indent_level;x++)
      putchar(' ');
  }

  if (type->is_flags == 0)
  {
    printf("%s", type->type);
    for (x = 0;x<type->pointerlevel;x++)
      putchar('*');

    return;
  }

  if (type->is_flags&IS_STRUCT)
  {
    struct _s_structdef *next;

    printf("%s", type->type);
    for (x = 0;x<type->pointerlevel;x++)
      putchar('*');

    next = type->extra_data;
    
    if (next != NULL)
    {
      printf(" {\n");

      while (next!=NULL)
      {
        output_type(&next->id, indent_level+2);
	printf(" %s; \n", next->id.name);

        next = next->next;
      }

      for (x=0;x<indent_level;x++)
        putchar(' ');
      printf("}");
    }
  }

  if (type->is_flags&IS_ENUM)
  {
    struct _s_enum *next;

    printf("%s", type->type);

    next = type->extra_data;
    if (next!=NULL)
    {
      printf(" {\n");
      for (x=0;x<indent_level;x++)
        putchar(' ');
    
      while (next!=NULL)
      {
        printf("  %s (%i)\n", next->name, next->number);
        for (x=0;x<indent_level;x++)
          putchar(' ');

        next = next->next;
      }

      printf("}");
    }
  }

  if (type->is_flags&IS_FUNCTION)
    {
      struct _s_functiondef *def;

      def = type->extra_data;
      output_type(&def->returntype, indent_level + 2);
      
      printf(" (");
      for (x=0;x<type->pointerlevel;x++)
	printf("*");
      printf("%s)\n", type->type);

      output_parameters(def->parameters);
    }
}

/* Structures to hold the parsed file */
struct _s_functionlist
{
  char *function_name;
  struct _s_functiondef  functiondef;

  struct _s_functionlist *next;
};

struct _s_typedeflist
{
  char *function_name;
  struct _s_typeidentifier type;

  struct _s_functionlist *next;
};

%}

/* Tokens */

%nonassoc WHITESPACE
%token <string> IDENTIFIER

%token <numeric> NUMBER

%token TYPEDEF
%token STRUCT
%token EXTERN
%token STATIC
%token ENUM
/* %token BASICTYPE */
%left POINTER

%token COMMENTSTART
%token DOCCOMMENTSTART
%token COMMENTEND

%token COMMENTBUMPH

%token MISC
%nonassoc VOID

/* Preprocessor stuff */

%token PREPRO
%token PREEOLN

%token PREDEFINE
%token PREIF
%token PREIFDEF
%token PREPRAGMA
%token PREENDIF
%token PREUNDEF
%token PREELSE

/* Doccomment stuff */

%token             DCEND
%nonassoc <string> DCTOKEN
%nonassoc <string> DCWORD
%left              DCWHITESPACE
  /* Note that you may get two DCWHITESPACEs in a row... */

/* Types of nonterminals */

%type <string>      BLANK
%type <string>      SCOPE
%type <type>        TYPE /* Type & identifier */
%type <type>        TYPEFUNC
%type <simpletype>  SIMPLETYPE /* Simple type */

%type <idlist>      IDLIST

%type <structdef>   STRUCTDEF
%type <structdef>   STRUCTCURLY

%type <enumt>       ENUMDEF
%type <enumt>       ENUMCURLY
%type <enumline>    ENUMLINE

%type <numeric>     POINT
%type <numeric>     POINTER
%type <numeric>     POINTERS

%type <parameters>  PARAMETERS
%type <parameters>  PARAMETER
%type <parameters>  PARAMETERLIST

%type <functiondecl> FUNCTIONDECL
%type <typedefdecl>  TYPEDEFDECL
%type <variabledecl> VARIABLEDECL
%type <headerfile>   HEADERFILE
%type <headerfile>   PARSER

%type <string>       DCSTRING

%start PARSER

%%

PARSER:   HEADERFILE BLANK 
          {
	    struct _s_headerfile *next;

	    printf("\n\nEnd of file...\n");

	    next = $1;
	    while (next != NULL)
	    {
	      printf("Item: ");
	      switch (next->headertype)
	      {
	      case DECL_FUNCTION:
	        printf("FUNCTION (%s)\n", next->data.function.type_name.name);
		break;

	      case DECL_VARIABLE:
	        printf("VARIABLE (%s)\n", next->data.variable.type_name.name);
		break;

	      case DECL_NAKEDSTRUCT:
	        printf("NAKED STRUCT/ENUM\n");
		break;

	      case DECL_TYPEDEF:
	        printf("TYPEDEF\n");
		break;

	      default:
	        printf("(UNKNOWN)\n");
	      }

	      next = next->next;
	    }
	  }

HEADERFILE:   /* Empty */                   
              {
	        $$ = NULL;
	      }
            | HEADERFILE FUNCTIONDECL
	      {
	        $$ = malloc(sizeof(struct _s_headerfile));

		$$->next          = $1;
		$$->headertype    = DECL_FUNCTION;
		$$->data.function = $2;
	      }
            | HEADERFILE VARIABLEDECL
	      {
	        $$ = malloc(sizeof(struct _s_headerfile));

		$$->next = $1;

		if ($2.type_name.name != NULL)
		{
		  $$->headertype    = DECL_VARIABLE;
		  $$->data.variable = $2;
		}
		else
		{
		  $$->headertype       = DECL_NAKEDSTRUCT;
		  $$->data.nakedstruct = $2;
		}
	      }
            | HEADERFILE TYPEDEFDECL
	      {
	        $$ = malloc(sizeof(struct _s_headerfile));

		$$->next             = $1;
		$$->headertype       = DECL_TYPEDEF;
		$$->data.typedefdecl = $2;
	      }
	    | HEADERFILE BLANK PREPROCESSOR
	      {
	        $$ = malloc(sizeof(struct _s_headerfile));

		$$ = $1;
	      }
	    | HEADERFILE COMMENT
	      {
	        $$ = $1;
	      }
            | HEADERFILE DOCCOMMENT
	      {
	        $$ = $1;
	      }
	    ;

/* Assorted bits & bobs */

/* Nothing, or whitespace */
BLANK:   /* Empty */ { $$ = ""; }
       | WHITESPACE  { $$ = " "; }
       ;

/* Variable/function scope (ie extern or static) */
SCOPE:   /* Empty */       { $$ = "extern"; }
       | EXTERN WHITESPACE { $$ = "extern" } 
       | STATIC WHITESPACE { $$ = "static" }
       ;

/* Type definition */
TYPE:   SIMPLETYPE POINT IDENTIFIER
	  {
	    $$.type         = $1.name;
	    $$.pointerlevel = $2;
	    $$.name         = $3;

	    $$.is_flags     = $1.is_flags;
	    $$.extra_data   = $1.extra_data;
	  }
      ;

/* Function type definition */
TYPEFUNC:   SIMPLETYPE POINT IDENTIFIER
	    {
	      $$.type         = $1.name;
	      $$.pointerlevel = $2;
	      $$.name         = $3;
	
  	      $$.is_flags     = $1.is_flags;
	      $$.extra_data   = $1.extra_data;
	    }
	  | SIMPLETYPE POINT '(' POINT IDENTIFIER BLANK ')' BLANK PARAMETERS 
            {
	      struct _s_functiondef *def;

	      $$.type         = STRING_NEWCOPY("function");
	      $$.pointerlevel = $4;
	      $$.name         = $5;

	      $$.is_flags     = IS_FUNCTION | HAS_NAME;
	      
	      def = malloc(sizeof(struct _s_functiondef));
	      def->parameters = $9;

	      def->returntype.type         = $1.name;
	      def->returntype.is_flags     = $1.is_flags;
	      def->returntype.pointerlevel = $2;
	      def->returntype.name         = NULL;
	      def->returntype.extra_data   = $1.extra_data;

	      $$.extra_data = def;
	    }
	  ;

SIMPLETYPE:  VOID 
	       {
	         $$.name = STRING_NEWCOPY("void");
		 $$.is_flags = 0;
	       }
      | IDENTIFIER { $$.name = $1; $$.is_flags = 0; }
      | STRUCT WHITESPACE IDENTIFIER STRUCTCURLY
          {
	    $$.name = malloc(strlen("struct ")+strlen($3)+1);
	    strcpy($$.name, "struct ");
	    strcat($$.name, $3);
	    $$.is_flags = IS_STRUCT;
	    $$.extra_data = $4;

	    if ($4 != NULL)
	      $$.is_flags |= HAS_NAME;
	  }
      | STRUCT WHITESPACE '{' BLANK STRUCTDEF BLANK '}'
          {
	    $$.name       = STRING_NEWCOPY("struct");
	    $$.is_flags   = IS_STRUCT;
	    $$.extra_data = $5;
	  }
      | ENUM WHITESPACE IDENTIFIER ENUMCURLY
          { 
            $$.name = malloc(strlen("enum ")+strlen($3)+1);
	    strcpy($$.name, "enum ");
	    strcat($$.name, $3);
	    $$.is_flags = IS_ENUM;
	    $$.extra_data = $4;

	    if ($4 != NULL)
	      $$.is_flags |= HAS_NAME;
	  }
      | ENUM WHITESPACE '{' BLANK ENUMDEF BLANK '}'
        { 
            $$.name = malloc(strlen("enum")+1);
	    strcpy($$.name, "enum");
	    $$.is_flags = IS_ENUM;
	    $$.extra_data = $5;
	}
      ;

/* Bits of a struct between curly brackets */

/* 
 * This definition and the ENUMCURLY defintion produce our shift/reduce 
 * conflict (WHITESPACE BLANK, y'know). Not a problem :-)
 */
STRUCTCURLY:   WHITESPACE                               { $$ = NULL }
             | WHITESPACE '{' BLANK STRUCTDEF BLANK '}' { $$ = $4 }
	     ;

IDLIST:   ',' BLANK IDENTIFIER
	    {
	      $$ = malloc(sizeof(struct _s_idlist));

	      $$->next = NULL;
	      $$->name = $3;
	    }
	| IDLIST BLANK ',' BLANK IDENTIFIER
	  {
	    $$ = malloc(sizeof(struct _s_idlist));
	    
	    $$->next = $1;
	    $$->name = $5;
	  }
	;

STRUCTDEF:   TYPEFUNC ';'
               {
	         $$ = malloc(sizeof(struct _s_structdef));

	         $$->next = NULL;
		 $$->id   = $1;
	       }
           | STRUCTDEF BLANK TYPEFUNC ';'
	     {
	       $$ = malloc(sizeof(struct _s_structdef));

	       $$->next = $1;
	       $$->id   = $3;
	     }
	   | TYPEFUNC IDLIST ';'
	     {
	       struct _s_idlist    *me, *next;
	       struct _s_structdef *def;

	       $$ = malloc(sizeof(struct _s_structdef));

	       $$->next = NULL;
	       $$->id   = $1;

	       /* Add duplicate type identifiers for each item */
	       me       = $2;

	       def      = $$;

	       while (me != NULL)
	       {
	         def->next = malloc(sizeof(struct _s_structdef));
		 def = def->next;

		 def->next    = NULL;
		 def->id      = $$->id;
		 def->id.name = me->name;

	         next = me->next;

		 free(me);
		 me = next;
	       }

	       def->next = $$;
	       next      = $$->next;
	       $$->next  = NULL;
	       $$        = next;
	     }
           | STRUCTDEF BLANK TYPEFUNC IDLIST ';'
	     {
	       struct _s_idlist    *me, *next;
	       struct _s_structdef *def;

	       $$ = malloc(sizeof(struct _s_structdef));

	       $$->next = $1;
	       $$->id   = $3;

	       /* Add duplicate type identifiers for each item */
	       me       = $4;

	       def      = $$;

	       while (me != NULL)
	       {
	         def->next = malloc(sizeof(struct _s_structdef));
		 def = def->next;

		 def->next    = $1;
		 def->id      = $$->id;
		 def->id.name = me->name;

	         next = me->next;

		 free(me);
		 me = next;
	       }

	       def->next = $$;
	       next      = $$->next;
	       $$->next  = $1;
	       $$        = next;
	     }
	   | QUICKCOMMENT 
	     { 
	       $$ = NULL;
	     }
	   | STRUCTDEF BLANK QUICKCOMMENT
	     {
	       $$ = $1;
	     }
           ;

/* Bits of an enum between curly brackets */
ENUMCURLY:   WHITESPACE                             { $$ = NULL; }
           | WHITESPACE '{' BLANK ENUMDEF BLANK '}' { $$ = $4; }
	   ;

ENUMDEF:   ENUMLINE 
	   { 
	     $$ = malloc(sizeof(struct _s_enum));

	     $$->number = 1;
	     $$->name   = $1.identifier;
	     if ($1.is_set)
	       $$->number = $1.number;
	     $$->next = NULL;
	   }
         | ENUMDEF BLANK ',' BLANK ENUMLINE
	   {
	     $$ = malloc(sizeof(struct _s_enum));

	     $$->next   = $1;
	     $$->name   = $5.identifier;
	     $$->number = $1->number+1;
	     if ($5.is_set)
	       $$->number = $5.number;
	   }
         | ENUMDEF BLANK ',' BLANK QUICKCOMMENT BLANK ENUMLINE
	   {
	     $$ = malloc(sizeof(struct _s_enum));

	     $$->next   = $1;
	     $$->name   = $7.identifier;
	     $$->number = $1->number+1;
	     if ($7.is_set)
	       $$->number = $7.number;
	   }
	 | ENUMDEF BLANK QUICKCOMMENT
	   {
	     $$ = $1;
	   }
	 | QUICKCOMMENT
	   {
	     $$ = NULL;
	   }
           ;

ENUMLINE:   IDENTIFIER BLANK                  { $$.is_set = 0; $$.identifier = $1 }
          | IDENTIFIER BLANK '=' BLANK NUMBER { $$.is_set = 1; $$.identifier = $1; $$.number = $5; }

/* Pointer '*'s */

POINT:   BLANK { $$ = 0; } 
       | BLANK '*' POINTERS { $$ = $3+1; }
       ;

POINTERS:   BLANK              { $$ = 0; }
          | POINTERS '*' BLANK { $$ = $1 + 1; }
          ;

/* Parameter list */

/* Note that VOID & WHITESPACE must be %nonassoc for this to work :-) */
PARAMETERS:   '(' BLANK ')'                     { $$ = NULL }
	    | '(' BLANK VOID BLANK ')'          { $$ = NULL }
	    | '(' BLANK PARAMETERLIST BLANK ')' { $$ = $3 }
            ;

PARAMETERLIST:   PARAMETER { $$ = $1; }
               | PARAMETERLIST BLANK ',' BLANK PARAMETER
                   {
		     $$ = $5;
		     $$->next = $1;
		   }
               ;

PARAMETER:   TYPE 
               {
	         $$ = malloc(sizeof(struct _s_param));
		 $$->next        = NULL;
		 $$->typepointer = $1;
	       }
           ;

/* Top level definitions (functions, etc) */

FUNCTIONDECL:   BLANK SCOPE TYPEFUNC BLANK PARAMETERS BLANK ';'
		  {
		    printf("Function: %s (type: %s, pointer level %i, scope %s)\n", $3.name, $3.type, $3.pointerlevel, $2);
		    output_parameters($5);

		    $$.type_name  = $3;
		    $$.parameters = $5;
		  }
              ;

VARIABLEDECL:   BLANK SCOPE TYPEFUNC BLANK ';' 
                  {
		    printf("Variable: ");
		    output_type(&$3, 0);
		    printf(" %s\n", $3.name);
		    
		    $$.type_name = $3;
		  }
	      | BLANK SCOPE SIMPLETYPE BLANK ';' 
	        {
		  if ($3.is_flags&IS_STRUCT && $3.is_flags&HAS_NAME)
		  {
		    $$.type_name.name       = NULL;
		    $$.type_name.type       = $3.name;
		    $$.type_name.is_flags   = $3.is_flags;
		    $$.type_name.extra_data = $3.extra_data;
		  }
		  else if ($3.is_flags&IS_ENUM)
		  {
		    $$.type_name.name       = NULL;
		    $$.type_name.type       = $3.name;
		    $$.type_name.is_flags   = $3.is_flags;
		    $$.type_name.extra_data = $3.extra_data;
		  }
		  else
		  {
		    yyerror("Parse error: Naked type");
		    YYERROR;
		  }
		}
              ;

/* Comment */

COMMENT:   BLANK COMMENTSTART WAFFLECOMMENT COMMENTEND { }
         ;

QUICKCOMMENT: COMMENTSTART WAFFLECOMMENT COMMENTEND { }

WAFFLECOMMENT:   COMMENTBUMPH
               | WAFFLECOMMENT COMMENTBUMPH
               ;

/* TYPEDEF */

TYPEDEFDECL:   BLANK TYPEDEF WHITESPACE TYPEFUNC BLANK ';' 
	     {
	       printf("Typedef: %s is now defined as:\n", $4.name);
	       output_type(&$4,2);
	       printf("\n");

	       $$.type_name = $4;
	     }
             ;

/* Preprocessor */

PREDIRECTIVE:   PREIF | PREIFDEF | PREPRAGMA | PREENDIF | PREUNDEF | PREELSE
              ;

PRESTUFF:   IDENTIFIER                { free($1); }
          | PRESTUFF BLANK IDENTIFIER { free($3); }
          ;

PREPARAMLIST:   BLANK IDENTIFIER                        { }
              | PREPARAMLIST BLANK ',' BLANK IDENTIFIER { }

PREDEFINEPARAM:   '(' BLANK ')'
                | '(' PREPARAMLIST ')'
                ;

PREPROCESSOR:   PREPRO BLANK PREDIRECTIVE BLANK PRESTUFF BLANK PREEOLN { }
              | PREPRO BLANK PREDIRECTIVE BLANK PREEOLN { }
              | PREPRO BLANK PREDEFINE WHITESPACE IDENTIFIER PREDEFINEPARAM BLANK PRESTUFF BLANK PREEOLN
              | PREPRO BLANK PREDEFINE WHITESPACE IDENTIFIER PREDEFINEPARAM BLANK PREEOLN
              | PREPRO BLANK PREDEFINE WHITESPACE IDENTIFIER WHITESPACE PRESTUFF BLANK PREEOLN
              | PREPRO BLANK PREDEFINE WHITESPACE IDENTIFIER BLANK PREEOLN
            ;

/* Documentation markup */

DOCCOMMENT: BLANK DOCCOMMENTSTART DCBODY DCEND { }
	    ;

DCBLANK:   /* Empty */
	 | DCWHITESPACE DCBLANK
	 ;

DCBODY:   DCSTRING                { printf("STRING - %s\n", $1); }
	| DCBODY DCTOKEN DCSTRING { printf("TOKEN/STRING - %s, %s\n", $2, $3); }
	;

DCSTRING:   DCWORD                { $$ = $1; }
	  | DCWHITESPACE          { $$ = NULL; }
	  | DCSTRING DCWORD       {
				    if ($1 != NULL)
				      {
				        $$ = malloc(strlen($1) + strlen($2) +
						    1);
					strcpy($$, $1);
					strcat($$, " ");
					strcat($$, $2);

					free($1); free($2);
				      }
				    else
				      {
				        $$ = $2;
				      }
				  }
	  | DCSTRING DCWHITESPACE { $$ = $1; }
	  ;

%%

int yyerror(char *s)
{
  printf("Doh! %s\n", s);
  abort();
}

int main(void)
{
  printf("Whoa!\n");
  yyparse();

  return 0;
}
